<?php
namespace App\Controller;

use Psr\Http\Message\ResponseInterface as Response;
use Psr\Http\Message\ServerRequestInterface as Request;
use \Datetime;

class Order {

    private $orderService;
    private $util;
    private $dateUtil;

    public function __construct(
        \App\Service\Order $orderService,
        \App\Util\Util $util,
        \App\Util\DateUtil $dateUtil
    ) {

        $this->orderService = $orderService;
        $this->util = $util;
        $this->dateUtil = $dateUtil;
    }

    public function history(Request $request, Response $response) {
        try {
            $params = $request->getParsedBody();
            if (empty($params['dtInicial']) || empty($params['dtFinal'])) {
                throw new \Exception("Data inicial/final não informada.");
            }

            $dataorder = $this->orderService->history($params);

            if(empty($dataorder)) {
                throw new \Exception("Nenhum pedido encontrado.");
            }

            foreach($dataorder as $row) {
                $row['IDORGCMDVENDA'] = $this->util->commandSourceDictionary($row['IDORGCMDVENDA']);
                $dataset[] = $row;
            }

            $response->getBody()->write(json_encode([
                'error' => false,
                'message' => '',
                'data' => $dataset,
                'pagination' => [
                    'limit' => $params['limit'] ?? 50,
                    'total' => count($dataset),
                    'page' => $params['page'] ?? 1
                ]
            ]));
    
            return $response->withHeader('Content-Type', 'application/json');
        } catch (\Exception $e) {
            $response->getBody()->write(json_encode([
                'error'   => true,
                'message' => $e->getMessage(),
                'data'    => []
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        }
    }

   
    public function cancel(Request $request, Response $response)
    {
        try {
            $params = $request->getParsedBody();
            $cdFilial    = !empty($params['CDFILIAL']) ? $params['CDFILIAL'] : null;
            $cdOperador  = !empty($params['CDOPERADOR']) ? $params['CDOPERADOR'] : null;
            $nrComanda   = !empty($params['NRCOMANDA']) ? $params['NRCOMANDA'] : null;
            $nrVendaRest = !empty($params['NRVENDAREST']) ? $params['NRVENDAREST'] : null;
            $dsCancMotiv = !empty($params['DSCANCMOTIV']) ? $params['DSCANCMOTIV'] : null;

            if(empty($cdFilial) || empty($cdOperador) || empty($nrComanda) || empty($nrVendaRest) || empty($dsCancMotiv)) {
                throw new \Exception("Algum parâmetro não foi informado.");
            }

            $resultCancel = $this->orderService->cancel($cdFilial, $cdOperador, $nrComanda, $nrVendaRest, $dsCancMotiv);

            $response->getBody()->write(json_encode([
                'error'   => false,
                'message' => $resultCancel,
                'data'    => []
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        } catch (\Exception $e) {
            $response->getBody()->write(json_encode([
                'error'   => true,
                'message' => $e->getMessage(),
                'data'    => []
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        }
    }

    public function getBranchList(Request $request, Response $response) {
        try {
            $params = $request->getParsedBody();
            $search = isset($params['search']) ? strval(strtoupper($params['search'])) : '';

            $branchList = $this->orderService->getBranchList();

            if(empty($branchList)) {
                throw new \Exception("Nenhuma filial encontrada.");
            }

            if (!empty($search) && $search !== '') {
                $branchList = array_values(array_filter($branchList, function($value) use ($search) {
                    return (mb_strpos($value['LABELFILIAL'], $search) !== false) || '';
                }));
            }

            $response->getBody()->write(json_encode([
                'error'   => false,
                'message' => '',
                'data'    => $branchList,
                'pagination' => [
                    'limit' => $params['limit'] ?? 10,
                    'total' => count($branchList),
                    'page' => $params['page'] ?? 1
                ]
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        } catch (\Exception $e) {
            $response->getBody()->write(json_encode([
                'error'   => true,
                'message' => $e->getMessage(),
                'data'    => []
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        }
    }

    public function getOrderNumber(Request $request, Response $response) {
        try {
            $params = $request->getQueryParams();
            $search = isset($params['search']) ? strval($params['search']) : '';
            $orderNumber = [];

            if (!empty($search) && strlen($search) > 4) {
                $orderNumber = $this->orderService->getOrderNumber($search);
            }

            $response->getBody()->write(json_encode([
                'error'   => false,
                'message' => '',
                'data'    => $orderNumber,
                'pagination' => [
                    'limit' => $params['limit'] ?? 10,
                    'total' => count($orderNumber),
                    'page' => $params['page'] ?? 1
                    ]
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        } catch (\Exception $e) {
            $response->getBody()->write(json_encode([
                'error'   => true,
                'message' => $e->getMessage(),
                'data'    => []
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        }
    }

    public function getConsumers(Request $request, Response $response) 
    {
        try {
            $params = $request->getQueryParams();

            $consumers = $this->orderService->getConsumers($params);

            if (isset($consumers)) {
            $response->getBody()->write(json_encode([
                'error'   => false,
                'message' => '',
                'data'    => $consumers,
                'pagination' => [
                    'limit' => $params['limit'] ?? 10,
                    'total' => count($consumers),
                    'page' => $params['page'] ?? 1
                ],
            ]));
        }
            return $response->withHeader('Content-Type', 'application/json');
        } catch (\Exception $e) {
            $response->getBody()->write(json_encode([
                'error'   => true,
                'message' => $e->getMessage(),
                'data'    => []
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        }
    }

    public function getOrderDetails(Request $request, Response $response) {
        try {
            $params = $request->getParsedBody();

            $orderDetails = $this->orderService->getOrderDetails($params);

            $response->getBody()->write(json_encode([
                'error' => false,
                'message' => '',
                'data' => $orderDetails
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        } catch (\Exception $e) {
            $response->getBody()->write(json_encode([
                'error'   => true,
                'message' => $e->getMessage(),
                'data'    => []
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        }
    }

    public function getOrderAddress(Request $request, Response $response) {
        try {
            $params = $request->getParsedBody();

            $orderAddress = $this->orderService->getOrderAddress($params);

            $response->getBody()->write(json_encode([
                'error' => false,
                'message' => '',
                'data' => $orderAddress
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        } catch (\Exception $e) {
            $response->getBody()->write(json_encode([
                'error'   => true,
                'message' => $e->getMessage(),
                'data'    => []
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        }
    }

    public function getPaymentMethods(Request $request, Response $response) {
        try {
            $params = $request->getParsedBody();

            $paymentMethods = $this->orderService->getPaymentMethods($params);
            
            $response->getBody()->write(json_encode([
                'error' => false,
                'message' => '',
                'data' => $paymentMethods
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        } catch (\Exception $e) {
            $response->getBody()->write(json_encode([
                'error'   => true,
                'message' => $e->getMessage(),
                'data'    => []
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        }
    }

    public function dashboard(Request $request, Response $response) {
        try {
            $params = $request->getParsedBody();

            $dtInicial = !empty($params['dtInicial']) ? $params['dtInicial'] : null;
            $dtFinal = !empty($params['dtFinal']) ? $params['dtFinal'] : null;

            if (empty($params['dtInicial']) || empty($params['dtFinal'])) {
                throw new \Exception("Data inicial/final não informada.");
            }
            
            $result = $this->orderService->dashboard($dtInicial, $dtFinal);
            $dataset = [
                'lastWeekDatasource' => [],
                'seriesInnerData' => [],
                'seriesOuterData' => [],
                'salesHour' => array_fill(0, 24, 0),
                'quantSalesToday' => 0,
                'valueSalesToday' => 0,
                'quantCancelToday' => 0,
                'valueCancelToday' => 0,
                'quantCancelCurrentMonth' => 0,
                'valueCancelCurrentMonth' => 0,
                'quantCancelLastMonth' => 0,
                'valueCancelLastMonth' => 0,
                'quantSalesCurrentMonth' => 0,
                'valueSalesCurrentMonth' => 0,
                'quantSalesLastMonth' => 0,
                'valueSalesLastMonth' => 0,
                'salesMonthModality' => [],
                'salesMonthModalityLegend' => [],
                'salesMonthModalityDatasource' => [],
                'salesMonthModalityData' => [],
                'salesMonthOrigin' => [],
                'salesMonthOriginLegend' => [],
                'salesMonthOriginDatasource' => [],
                'salesMonthOriginData' => [],
                'cancelMonthOrigin' => [],
                'cancelMonthOriginLegend' => [],
                'cancelMonthOriginDatasource' => [],
                'cancelMonthOriginData' => []
            ];
            $quantSalesLastWeek = [];
            $valueSalesLastWeek = [];
            $todayInnerData = [];
            $todayOuterData = [];
            $foundNonTodayOrder = false;
            $foundNonCurrentWeekOrder = false;
            $foundNonCurrentMonthOrder = false;
            foreach ($result as $row) {
                if ($row['IDSTCOMANDA'] === 'A') {
                    continue;
                }

                $row['IDORGCMDVENDA'] = $this->util->commandSourceDictionary($row['IDORGCMDVENDA']);
            
                if ($row['IDSTCOMANDA'] !== '4') {
                    $this->processMonthlySalesData($row, $dataset, 'salesMonthModality', 'TOGO');
                    $this->processMonthlySalesData($row, $dataset, 'salesMonthOrigin', 'IDORGCMDVENDA');
        
                    if(!$foundNonTodayOrder && $this->dateUtil->isToday($row['DTHRABERMESA'])) {
                        $todayInnerData[$row['TOGO']] = ($todayInnerData[$row['TOGO']] ?? 0) + 1;
                        $todayOuterData[$row['IDORGCMDVENDA']] = ($todayOuterData[$row['IDORGCMDVENDA']] ?? 0) + 1;

                        $saleHour = ltrim(DateTime::createFromFormat('d/m/Y H:i:s', $row['DTHRABERMESA'])->format('H'), '0');
                        $dataset['salesHour'][$saleHour] += 1;

                        $dataset['quantSalesToday'] += 1;
                        $dataset['valueSalesToday'] += $row['VRPEDIDO'];

                        $dataset['quantSalesCurrentMonth'] += 1;
                        $dataset['valueSalesCurrentMonth'] += $row['VRPEDIDO'];
                    }
                    else {
                        $foundNonTodayOrder = true;
                        if(!$foundNonCurrentMonthOrder && $this->dateUtil->isCurrentMonth($row['DTHRABERMESA'])) {
                            $dataset['quantSalesCurrentMonth'] += 1;
                            $dataset['valueSalesCurrentMonth'] += $row['VRPEDIDO'];
                        }
                        else {
                            $foundNonCurrentMonthOrder = true;
                            $dataset['quantSalesLastMonth'] += 1;
                            $dataset['valueSalesLastMonth'] += $row['VRPEDIDO'];
                        }
                        if(!$foundNonCurrentWeekOrder && $this->dateUtil->isLastWeek($row['DTHRABERMESA'])) {
                            $day = DateTime::createFromFormat('d/m/Y H:i:s', $row['DTHRABERMESA'])->format('d/m');
        
                            $quantSalesLastWeek[$day] = ($quantSalesLastWeek[$day] ?? 0) + 1;
                            $valueSalesLastWeek[$day] = ($valueSalesLastWeek[$day] ?? 0) + $row['VRPEDIDO'];
                        }
                        else {
                            $foundNonCurrentWeekOrder = true;
                        }
                    }
                } else {
                    $this->processMonthlySalesData($row, $dataset, 'cancelMonthOrigin', 'IDORGCMDVENDA');
                    
                    if(!$foundNonTodayOrder && $this->dateUtil->isToday($row['DTHRABERMESA'])) {
                        $dataset['quantCancelToday'] += 1;
                        $dataset['valueCancelToday'] += $row['VRPEDIDO'];
                        $dataset['quantCancelCurrentMonth'] += 1;
                        $dataset['valueCancelCurrentMonth'] += $row['VRPEDIDO'];
                    }
                    else {
                        $foundNonTodayOrder = true;
                        if(!$foundNonCurrentMonthOrder && $this->dateUtil->isCurrentMonth($row['DTHRABERMESA'])) {
                            $dataset['quantCancelCurrentMonth'] += 1;
                            $dataset['valueCancelCurrentMonth'] += $row['VRPEDIDO'];
                        }
                        else {
                            $foundNonCurrentMonthOrder = true;
                            $dataset['quantCancelLastMonth'] += 1;
                            $dataset['valueCancelLastMonth'] += $row['VRPEDIDO'];
                        }
                    }
                }
            }
            $this->getDatasource($dataset, 'salesMonthModality');
            $this->getDatasource($dataset, 'salesMonthOrigin');
            $this->getDatasource($dataset, 'cancelMonthOrigin'); 
            foreach ($todayInnerData as $modality => $quant) {
                $dataset['seriesInnerData'][] = [$modality . ' (' . $quant . ')', $quant];
            }
            foreach ($todayOuterData as $origin => $quant) {
                $dataset['seriesOuterData'][] = [$origin . ' (' . $quant . ')', $quant];
            }
            $day = new DateTime('yesterday');
            $weekDays = 7;
            for ($i=0; $i<$weekDays; $i++) {
                $date = $day->format('d/m');
                $dataset['lastWeekDatasource'][$i] = [$date, $quantSalesLastWeek[$date] ?? 0, 
                    number_format($valueSalesLastWeek[$date] ?? 0, 2, '.', '')];
                $day->modify("-1 day");    
            }

            $response->getBody()->write(json_encode([
                'error' => false,
                'message' => "Dados do Dashboard adquiridos com sucesso.",
                'data' => $dataset
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        } catch (\Exception $e) {
            $response->getBody()->write(json_encode([
                'error'   => true,
                'message' => $e->getMessage(),
                'data'    => []
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        }
    }

    private function processMonthlySalesData($row, &$dataset, $name, $type) {
        $data = DateTime::createFromFormat('d/m/Y H:i:s', $row['DTHRABERMESA']);
        $month = $data->format('n');
        if (!isset($dataset[$name][$month])) {
            $dataset[$name][$month] = [];
        }
        $modality = $row[$type];
        if (!isset($dataset[$name][$month][$modality])) {
            $dataset[$name][$month][$modality] = 0;
        }
        if (!in_array($row[$type], $dataset[$name.'Legend'])) {
            $dataset[$name.'Legend'][] = $row[$type];
            $newSeries = [
                'name' => $row[$type],
                'type' => 'line',
                'data' => array_fill(0, 12, 0),
                'label' => [
                    'show' => true,
                    'position' => 'top',
                    'color' => 'black',
                    'fontSize' => 12,
                    'formatter' => function($d) {
                        return $d['data'];
                    },
                ],
            ];
            $dataset[$name.'Datasource'][] = $newSeries;
        }
        $dataset[$name][$month][$modality] += 1;
    }

    private function getDatasource(&$dataset, $name) {
        $retroMonths = 6;
        $dataset[$name.'Data'] = $this->dateUtil->getPreviousMonths($retroMonths, true);
        $todayMonth = date('n');
        $nextMonth = new DateTime();
        $nextMonth->setDate($nextMonth->format('Y'), $nextMonth->format('m'), 1);
        $nextMonth->modify('-' . $retroMonths . ' months');
        $k = 0;
        while (intval($nextMonth->format('n')) !== intval($todayMonth + 1)) {
            $j = 0;
            $totalSalesMonth = 0;
            for ($j; $j < count($dataset[$name.'Legend']); $j++) {
                if (isset($dataset[$name][$nextMonth->format('n')])) {
                    $dataset[$name.'Datasource'][$j]['data'][$k] = $dataset[$name][$nextMonth->format('n')][$dataset[$name.'Legend'][$j]] ?? 0;
                    $totalSalesMonth += $dataset[$name][$nextMonth->format('n')][$dataset[$name.'Legend'][$j]] ?? 0;
                }
            }

            $dataset[$name.'Data'][$k] = $dataset[$name.'Data'][$k] . '(' . $totalSalesMonth . ')';
            $nextMonth->modify('+1 month');
            $k += 1;
        }
    }

    public function getOrderItemsRepo(Request $request, Response $response) {
        try {
			$params   = $request->getParsedBody();
            $products = $this->orderService->getOrderItemsRepo($params);

            $response->getBody()->write(json_encode([
                'error' => false,
                'message' => '',
                'data' => $products
            ]));

            return $response->withHeader('Content-Type', 'application/json');
		} catch (\Exception $e) {
            $response->getBody()->write(json_encode([
                'error'   => true,
                'message' => $e->getMessage(),
                'data'    => []
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        }
    }

    public function getSubItensRepo(Request $request, Response $response) {
        try {
            $params = $request->getParsedBody();

            $subItens = $this->orderService->getSubItens($params);

            $response->getBody()->write(json_encode([
                'error' => false,
                'message' => '',
                'data' => $subItens
            ]));

            return $response->withHeader('Content-Type', 'application/json');
		} catch (\Exception $e) {
            $response->getBody()->write(json_encode([
                'error'   => true,
                'message' => $e->getMessage(),
                'data'    => []
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        }
    }

    public function getItemObservation(Request $request, Response $response) {
        try {
            $params = $request->getParsedBody();
            $itemObservation = $this->orderService->getItemObservation($params);

            $response->getBody()->write(json_encode([
                'error' => false,
                'message' => '',
                'data' => $itemObservation
            ]));

            return $response->withHeader('Content-Type', 'application/json');
		} catch (\Exception $e) {
            $response->getBody()->write(json_encode([
                'error'   => true,
                'message' => $e->getMessage(),
                'data'    => []
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        }
    }

    public function forwardOrder(Request $request, Response $response) {
        try {
            $params = $request->getParsedBody();

            $cdFilial   = !empty($params['CDFILIAL']) ? $params['CDFILIAL'] : null;
            $nrComanda = !empty($params['NRCOMANDA']) ? $params['NRCOMANDA'] : null;
            $nrVendaRest = !empty($params['NRVENDAREST']) ? $params['NRVENDAREST'] : null;

            if(empty($cdFilial) || empty($nrComanda) || empty($nrVendaRest)) {
                throw new \Exception("Algum parâmetro não foi informado.");
            }
  
            $forwardOrder = $this->orderService->forwardOrder($cdFilial, $nrComanda, $nrVendaRest);
  
            $response->getBody()->write(json_encode([
                'error' => false,
                'message' => 'Pedido adiantado com sucesso.',
                'data' => $forwardOrder
            ]));

            return $response->withHeader('Content-Type', 'application/json');
          } catch (\Exception $e) {
            $response->getBody()->write(json_encode([
                'error'   => true,
                'message' => $e->getMessage(),
                'data'    => []
            ]));

            return $response->withHeader('Content-Type', 'application/json');
        }
      }
  
}